#//-----------------------------------------------
#//TUSB-INS8/8用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_05_01　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

tusbins8dll = cdll.LoadLibrary("TUSBINS8.DLL")

tusbins8dll.Tusbins88_Device_Open.restype = c_short
tusbins8dll.Tusbins88_Device_Open.argtypes = (c_short,)
tusbins8dll.Tusbins88_Device_Close.restype = None
tusbins8dll.Tusbins88_Device_Close.argtypes = (c_short,)
tusbins8dll.Tusbins88_Write.restype = c_short
tusbins8dll.Tusbins88_Write.argtypes = (c_short,c_ubyte)
tusbins8dll.Tusbins88_Read.restype = c_short
tusbins8dll.Tusbins88_Read.argtypes = (c_short,POINTER(c_ubyte))
tusbins8dll.Tusbins88_EdgeSet.restype = c_short
tusbins8dll.Tusbins88_EdgeSet.argtypes = (c_short,c_ubyte,c_ubyte)
tusbins8dll.Tusbins88_EdgeRead.restype = c_short
tusbins8dll.Tusbins88_EdgeRead.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (don not call directry from user codes)

def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):   return False
    elif(Val < Min):            return False
    elif(Val > Max):            return False
    else:                       return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

# def functions ------------------------------------

def Tusbins88_Device_Open(DevId):
    """Open Device Connection
    
    Arguments:
    DevId           ----    DevIce ID 0~15
    
    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 1
    return tusbins8dll.Tusbins88_Device_Open(DevId)

def Tusbins88_Device_Close(DevId):
    """Close Device Connection

    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    None
    """
    if(not IsValidId(DevId)):return 2
    tusbins8dll.Tusbins88_Device_Close(DevId)

def Tusbins88_Write(DevId,Data):
    """Write Output Resistor
    
    Arguments:
    DevId           ----    Device ID 0~15
    Deta            ----    Output Data bit pattern 0~255

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 2
    if(not IsInRange(Data,0,255)):return 2
    return tusbins8dll.Tusbins88_Write(DevId,Data)

def Tusbins88_Read(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int,            ----    Result code
    int,            ----    Input Data
    """
    if(not IsValidId(DevId)):return 2
    Data = c_ubyte()
    ret = tusbins8dll.Tusbins88_Read(DevId,byref(Data))
    if ret == 0:
        return 0,Data.value
    else:
        return ret,None

def Tusbins88_EdgeSet(DevId,Data,Time):
    """Set Edge Detector Settings
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data            ----    Edge Direction bits
    Time            ----    Detection time 0~255

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 2
    if(not IsInRange(Data,0,255)):return 2
    if(not IsInRange(Time,0,255)):return 2
    return tusbins8dll.Tusbins88_EdgeSet(DevId,Data,Time)

def Tusbins88_EdgeRead(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int,            ----    Result code
    int,            ----    Input Data
    """
    if(not IsValidId(DevId)):return 2
    Data = c_ubyte()
    ret = tusbins8dll.Tusbins88_EdgeRead(DevId,byref(Data))
    if ret == 0:
        return 0,Data.value
    else:
        return ret,None

#戻り値用エラーメッセージ(オープン関数のみ)
OpenErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
}
def GetOpenErrMessage(retcode):
    if( retcode in OpenErrStrDic):
        return OpenErrStrDic[retcode]
    else:
        return "不明なエラーです"

#戻り値用エラーメッセージ
ErrStrDic = {
            0:"正常終了しました",
            1:"このデバイスはオープンされていません",
            2:"失敗しました",
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"
